import { Inject, Injectable } from '@nestjs/common';
import { Response } from 'express';
import { v4 } from 'uuid';
import { CACHE_MANAGER } from '@nestjs/cache-manager';
import { RedisCache } from 'cache-manager-redis-yet';
import { IRequest } from '@/interface';
import { config } from '../../config';

// const EXPIRE_TIME_IN_SEC = 86400;
const EXPIRE_TIME_IN_SEC = 3600;

export function readKeyFromCookie(
  type: string,
  req: IRequest,
  throwError = true,
) {
  const key = req.cookies[type];
  if (!key && throwError) {
    throw new Error('Without ' + type);
  }
  return key;
}
export function saveKeyIntoCookie(
  type: string,
  key: string,
  res: Response,
  options: {
    timeout?: number;
  } = {},
) {
  const { timeout = EXPIRE_TIME_IN_SEC } = options;
  res.cookie(type, key, {
    maxAge: timeout * 1000,
    secure: true,
    sameSite: 'none',
    httpOnly: true,
    signed: true,
  });
}

export async function clearKeyFromCookie(type: string, res: Response) {
  res.clearCookie(type, {
    // domain: getSubDomainHost('').split(':')[0],
  });
}

/**
 * 负责在顶级域名下存储一次性的临时 session，用来在流程中的各端点之间传递状态信息
 * 将redis的key保存在cookie中，将js对象转为string生成redis键值对
 */
@Injectable()
export class CacheService {
  constructor(@Inject(CACHE_MANAGER) private cacheManager: RedisCache) {}

  async setNx<T = Record<string, any>>(
    type: string,
    key: string,
    value: T,
    options: {
      timeout?: number;
    } = {},
  ) {
    const { timeout = EXPIRE_TIME_IN_SEC } = options;

    await this.cacheManager.store.client.set(
      config.env + ':' + type + ':' + key,
      JSON.stringify(value),
      // 超时秒数
      { EX: timeout, NX: true },
    );
  }

  /**
   * 创建一个 Interaction，存储到 redis 中。
   *
   * @param type 类型
   * @param value 值
   * @param options 额外参数
   *  - timeout：过期时间，默认为 86400 秒
   *  - key：唯一标识，不传会自动生成
   * @returns Interaction 的唯一标识
   */
  async create<T = Record<string, any>>(
    type: string,
    key: string,
    value: T,
    options: {
      timeout?: number;
    } = {},
  ) {
    const { timeout = EXPIRE_TIME_IN_SEC } = options;

    await this.cacheManager.store.client.set(
      config.env + ':' + type + ':' + key,
      JSON.stringify(value),
      // 超时秒数
      { EX: timeout },
    );

    return key;
  }

  /**
   * 创建一个 Interaction，存储到 cookie 和 redis 中。
   *
   * @param type 类型
   * @param value 值
   * @param res 响应对象
   * @param options 额外参数
   *  - timeout：过期时间，默认为 86400 秒
   *  - key：唯一标识，不传会自动生成`
   * @returns Interaction 的唯一标识
   */
  async createIntoCookie<T = Record<string, any>>(
    type: string,
    value: T,
    res: Response,
    options: {
      timeout?: number;
      key?: string;
    } = {},
  ) {
    const key = options.key ?? v4();

    await this.create(type, key, value, options);
    saveKeyIntoCookie(type, key, res, options);
    return key;
  }

  /**
   * 更新一个 Interaction 的值。
   *
   * @param type 类型
   * @param value 值
   * @param key 唯一标识
   */
  async update<T = Record<string, any>>(
    type: string,
    key: string,
    value: T,
    options: {
      timeout?: number;
    } = {},
  ) {
    await this.cacheManager.set(
      config.env + ':' + type + ':' + key,
      JSON.stringify(value),
      options.timeout ?? EXPIRE_TIME_IN_SEC,
    );
  }

  /**
   * 读取缓存值。
   *
   * @param type 类型
   * @param key 唯一标识
   * @returns Interaction 的值
   */
  async read<T = Record<string, any>>(type: string, key: string) {
    const value = await this.cacheManager.get<T>(
      config.env + ':' + type + ':' + key,
    );

    return value;
  }

  /**
   * 清除一个 Interaction。
   *
   * @param type 类型
   * @param key 唯一标识
   */
  async clear(type: string, key: string) {
    await this.cacheManager.set(config.env + ':' + type + ':' + key, 10);
  }
}
