import {
  ArgumentsHost,
  BadRequestException,
  Catch,
  ExceptionFilter,
  ForbiddenException,
  Injectable,
  UnauthorizedException,
} from '@nestjs/common';
import { Response } from 'express';
import { IRequest } from '@/interface';
import { isArray, isString } from 'lodash';

@Injectable()
@Catch()
export class ApiExceptionsFilter implements ExceptionFilter {
  catch(exception: any, host: ArgumentsHost) {
    const ctx = host.switchToHttp();
    const response = ctx.getResponse<Response>();
    const req = ctx.getRequest<IRequest>();

    if (exception instanceof ForbiddenException) {
      return response.status(403).json({
        code: 403,
        message: 'Not Permission',
      });
    }

    if (exception instanceof UnauthorizedException) {
      const statusCode = (exception as any)?.response?.statusCode || 400;
      return response.status(401).json({
        code: statusCode,
        message: (exception as any)?.response?.message || 'Bad Request',
      });
    }

    if (exception instanceof BadRequestException) {
      const validationErrors = (exception as any)?.response?.message;

      const statusCode = (exception as any)?.response?.statusCode || 400;

      if (isArray(validationErrors) && validationErrors?.length) {
        return response.status(statusCode).json({
          code: statusCode,
          message: validationErrors[0],
        });
      } else {
        return response.status(statusCode).json({
          code: statusCode,
          message: (exception as any)?.response?.message || 'Bad Request',
        });
      }
    }

    // 未知错误，触发错误报警
    console.error('请求 HOST：' + req?.hostname);
    console.error('请求路径：' + JSON.stringify(req?.path));
    console.error('请求参数 QUERY:' + JSON.stringify(req?.query));
    console.error('请求参数 BODY:' + JSON.stringify(req?.body));

    console.error(
      '\n***\n',
      '未知错误',
      JSON.stringify(exception?.message ?? exception),
    );
    const status = exception?.status || 500;

    console.error('requestId:' + req.requestId);

    return response.status(status || 500).json({
      code: status,
      message: isString(exception.message)
        ? exception.message
        : JSON.parse(exception.message),
      requestId: req.requestId,
    });
  }
}
